@@ -1082,7 +1082,7 @@ agent_keytotpm (ctrl_t ctrl, const char *hexgrip)
 
   if (strchr (hexgrip, ','))
     {
-      log_error ("storing a part of a dual key is not yet supported\n");
+      log_error ("storing a part of a composite key is not yet supported\n");
       return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
     }
 
@@ -1117,7 +1117,7 @@ agent_keytocard (const char *hexgrip, int keyno, int force,
 
   if (strchr (hexgrip, ','))
     {
-      log_error ("storing a part of a dual key is not yet supported\n");
+      log_error ("storing a part of a composite key is not yet supported\n");
       return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
     }
 
@@ -2434,9 +2434,14 @@ agent_probe_any_secret_key (ctrl_t ctrl, kbnode_t keyblock)
             nkeys++;
 
             err = keygrip_from_pk (node->pkt->pkt.public_key, grip2, 1);
-            if (err && gpg_err_code (err) != GPG_ERR_FALSE)
-              return err;
-            if (!err)  /* Add the second keygrip from dual algos.  */
+            if (err)
+              {
+                if (gpg_err_code (err) == GPG_ERR_FALSE) /* No second keygrip.  */
+                  err = 0;
+                else
+                  return err;
+              }
+            else /* Add the second keygrip from dual algos.  */
               {
                 *p++ = ' ';
                 bin2hex (grip2, 20, p);
@@ -2479,8 +2484,6 @@ agent_get_keyinfo (ctrl_t ctrl, const char *hexkeygrip,
   if (err)
     return err;
 
-  /* FIXME: Support dual keys.  Maybe under the assumption that the
-   *        first key might be on a card.  */
   if (!hexkeygrip)
     return gpg_error (GPG_ERR_INV_VALUE);
   s = strchr (hexkeygrip, ',');
@@ -2668,6 +2671,30 @@ agent_genkey (ctrl_t ctrl, char **cache_nonce_addr, char **passwd_nonce_addr,
 }
 
 
+/* Add the Link attribute to both given keys.  */
+gpg_error_t
+agent_crosslink_keys (ctrl_t ctrl, const char *hexgrip1, const char *hexgrip2)
+{
+  gpg_error_t err;
+  char line[ASSUAN_LINELENGTH];
+
+  err = start_agent (ctrl, 0);
+  if (err)
+    goto leave;
+
+  snprintf (line, sizeof line, "KEYATTR %s Link: %s", hexgrip1, hexgrip2);
+  err = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
+  if (err)
+    goto leave;
+
+  snprintf (line, sizeof line, "KEYATTR %s Link: %s", hexgrip2, hexgrip1);
+  err = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
+
+ leave:
+  return err;
+}
+
+
 
 /* Call the agent to read the public key part for a given keygrip.
  * Values from FROMCARD:
@@ -2880,6 +2907,7 @@ agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
   char *p, *buf, *endp;
   const char *keygrip2 = NULL;
   struct default_inq_parm_s dfltparm;
+  const char *cmdline;
 
   memset (&dfltparm, 0, sizeof dfltparm);
   dfltparm.ctrl = ctrl;
@@ -2906,6 +2934,12 @@ agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
         return gpg_error (GPG_ERR_INV_VALUE);
     }
 
+  if (*keygrip2)
+    cmdline = "PKDECRYPT --kem=PQC-PGP";
+  else if (pubkey_algo  == PUBKEY_ALGO_ECDH)
+    cmdline = "PKDECRYPT --kem=PGP";
+  else
+    cmdline = "PKDECRYPT";
 
   err = start_agent (ctrl, 0);
   if (err)
@@ -2948,8 +2982,7 @@ agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
     err = make_canon_sexp (s_ciphertext, &parm.ciphertext, &parm.ciphertextlen);
     if (err)
       return err;
-    err = assuan_transact (agent_ctx,
-                           *keygrip2? "PKDECRYPT --kem=PQC-PGP":"PKDECRYPT",
+    err = assuan_transact (agent_ctx, cmdline,
                            put_membuf_cb, &data,
                            inq_ciphertext_cb, &parm,
                            padding_info_cb, r_padding);
@@ -3071,7 +3104,8 @@ inq_import_key_parms (void *opaque, const char *line)
 
 /* Call the agent to import a key into the agent.  */
 gpg_error_t
-agent_import_key (ctrl_t ctrl, const char *desc, char **cache_nonce_addr,
+agent_import_key (ctrl_t ctrl, const char *desc, int mode1003,
+                  char **cache_nonce_addr,
                   const void *key, size_t keylen, int unattended, int force,
 		  u32 *keyid, u32 *mainkeyid, int pubkey_algo, u32 timestamp)
 {
@@ -3093,6 +3127,12 @@ agent_import_key (ctrl_t ctrl, const char *desc, char **cache_nonce_addr,
     return err;
   dfltparm.ctx = agent_ctx;
 
+  /* Check that the gpg-agent supports the --mode1003 option.  */
+  if (mode1003 && assuan_transact (agent_ctx,
+                                   "GETINFO cmd_has_option IMPORT_KEY mode1003",
+                                   NULL, NULL, NULL, NULL, NULL, NULL))
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Do not use our cache of secret keygrips anymore - this command
    * would otherwise requiring to update that cache.  */
   if (ctrl && ctrl->secret_keygrips)
@@ -3122,9 +3162,10 @@ agent_import_key (ctrl_t ctrl, const char *desc, char **cache_nonce_addr,
   parm.key    = key;
   parm.keylen = keylen;
 
-  snprintf (line, sizeof line, "IMPORT_KEY%s%s%s%s%s",
+  snprintf (line, sizeof line, "IMPORT_KEY%s%s%s%s%s%s",
             *timestamparg? timestamparg : "",
             unattended? " --unattended":"",
+            mode1003? " --mode1003":"",
             force? " --force":"",
             cache_nonce_addr && *cache_nonce_addr? " ":"",
             cache_nonce_addr && *cache_nonce_addr? *cache_nonce_addr:"");
@@ -3193,7 +3234,7 @@ agent_export_key (ctrl_t ctrl, const char *hexkeygrip, const char *desc,
 
   snprintf (line, DIM(line), "EXPORT_KEY %s%s%s %s",
             mode1003? "--mode1003" : openpgp_protected ? "--openpgp ":"",
-            cache_nonce_addr && *cache_nonce_addr? "--cache-nonce=":"",
+            cache_nonce_addr && *cache_nonce_addr? " --cache-nonce=":"",
             cache_nonce_addr && *cache_nonce_addr? *cache_nonce_addr:"",
             hexkeygrip);
 
@@ -3279,7 +3320,7 @@ agent_delete_key (ctrl_t ctrl, const char *hexkeygrip, const char *desc,
         return err;
     }
 
-  /* FIXME: Shall we add support to DELETE_KEY for dual keys?  */
+  /* FIXME: Shall we add support to DELETE_KEY for composite keys?  */
   snprintf (line, DIM(line), "DELETE_KEY%s %s",
             force? " --force":"", hexkeygrip);
   err = assuan_transact (agent_ctx, line, NULL, NULL,
